// Copyright 2010 Washington University School of Medicine All Rights Reserved
package org.nrg.xnat.restlet.resources;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.nrg.xdat.base.BaseElement;
import org.nrg.xdat.om.XnatExperimentdata;
import org.nrg.xdat.om.XnatExperimentdataShare;
import org.nrg.xdat.om.XnatImageassessordata;
import org.nrg.xdat.om.XnatProjectdata;
import org.nrg.xdat.om.XnatSubjectdata;
import org.nrg.xdat.schema.SchemaElement;
import org.nrg.xdat.security.SecurityValues;
import org.nrg.xft.XFTItem;
import org.nrg.xft.XFTTable;
import org.nrg.xft.db.MaterializedView;
import org.nrg.xft.db.ViewManager;
import org.nrg.xft.exception.DBPoolException;
import org.nrg.xft.exception.InvalidValueException;
import org.nrg.xft.schema.Wrappers.GenericWrapper.GenericWrapperElement;
import org.nrg.xft.schema.design.SchemaElementI;
import org.nrg.xft.search.CriteriaCollection;
import org.nrg.xft.search.QueryOrganizer;
import org.nrg.xft.security.UserI;
import org.nrg.xft.utils.StringUtils;
import org.nrg.xft.utils.ValidationUtils.ValidationResults;
import org.restlet.Context;
import org.restlet.data.MediaType;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.Representation;
import org.restlet.resource.Variant;

public class ProjSubExptAsstList extends QueryOrganizerResource {
	XnatProjectdata proj=null;
	XnatSubjectdata sub=null;
	XnatExperimentdata assessed=null;
	
	public ProjSubExptAsstList(Context context, Request request, Response response) {
		super(context, request, response);
		
			String pID= (String)request.getAttributes().get("PROJECT_ID");
			if(pID!=null){
				proj = XnatProjectdata.getXnatProjectdatasById(pID, user, false);
				
			if (proj == null) {
				ArrayList<XnatProjectdata> matches = XnatProjectdata
						.getXnatProjectdatasByField(
								"xnat:projectData/aliases/alias/alias", pID,
								user, false);
				if (matches.size() > 0) {
					proj = matches.get(0);
				}
			}

				String subID= (String)request.getAttributes().get("SUBJECT_ID");
				if(subID!=null){
				sub = XnatSubjectdata.GetSubjectByProjectIdentifier(proj
						.getId(), subID, user, false);
					
					if(sub==null){
					sub = XnatSubjectdata.getXnatSubjectdatasById(subID, user,
							false);
					if (sub != null
							&& (proj != null && !sub.hasProject(proj.getId()))) {
						sub = null;
					}
					}
					
					if(sub!=null){
					String exptID = (String) request.getAttributes().get(
							"ASSESSED_ID");
					assessed = XnatExperimentdata.getXnatExperimentdatasById(
							exptID, user, false);
					if (assessed != null
							&& (proj != null && !assessed.hasProject(proj
									.getId()))) {
						assessed = null;
					}
						
						if(assessed==null){
						assessed = XnatExperimentdata
								.GetExptByProjectIdentifier(proj.getId(),
										exptID, user, false);
						}
						
						if(assessed!=null){
						this.getVariants().add(
								new Variant(MediaType.APPLICATION_JSON));
						this.getVariants()
								.add(new Variant(MediaType.TEXT_HTML));
							this.getVariants().add(new Variant(MediaType.TEXT_XML));
						}else{
						response.setStatus(Status.CLIENT_ERROR_NOT_FOUND,
								"Unable to find experiment.");
					}
				}else{
					response.setStatus(Status.CLIENT_ERROR_NOT_FOUND,
							"Unable to find subject.");
				}
			}else{
				response.setStatus(Status.CLIENT_ERROR_NOT_FOUND,
						"Unable to find subject.");
			}
		}else{
			response.setStatus(Status.CLIENT_ERROR_NOT_FOUND,
					"Unable to find project.");
		}

		this.fieldMapping.put("ID", "xnat:experimentdata/ID");
		this.fieldMapping.put("visit_id", "xnat:experimentdata/visit_id");
		this.fieldMapping.put("date", "xnat:experimentdata/date");
		this.fieldMapping.put("time", "xnat:experimentdata/time");
		this.fieldMapping.put("note", "xnat:experimentdata/note");
		this.fieldMapping.put("pi_firstname", "xnat:experimentdata/investigator/firstname");
		this.fieldMapping.put("pi_lastname", "xnat:experimentdata/investigator/lastname");
		this.fieldMapping.put("validation_method", "xnat:experimentdata/validation/method");
		this.fieldMapping.put("validation_status", "xnat:experimentdata/validation/status");
		this.fieldMapping.put("validation_date", "xnat:experimentdata/validation/date");
		this.fieldMapping.put("validation_notes", "xnat:experimentdata/validation/notes");
		this.fieldMapping.put("project", "xnat:experimentdata/project");
		this.fieldMapping.put("label", "xnat:experimentdata/label");
		
		this.fieldMapping.put("session_ID", "xnat:imagesessiondata/id");
		this.fieldMapping.put("session_label", "xnat:imagesessiondata/label");
		this.fieldMapping.put("session_project", "xnat:imagesessiondata/project");
		
		this.fieldMapping.put("insert_date", "xnat:experimentData/meta/insert_date");
		this.fieldMapping.put("insert_user", "xnat:experimentData/meta/insert_user/login");
		this.fieldMapping.put("last_modified", "xnat:experimentData/meta/last_modified");
		this.fieldMapping.put("xsiType", "xnat:experimentData/extension_item/element_name");
	}


	@Override
	public boolean allowPost() {
		return true;
	}

	@Override
	public void handlePost() {
	        XFTItem item = null;			

			try {
			item=this.loadItem(null,true);
			
				if(item==null){
					String xsiType=this.getQueryVariable("xsiType");
					if(xsiType!=null){
						item=XFTItem.NewItem(xsiType, user);
					}
				}
				
				if(item==null){
					this.getResponse().setStatus(Status.CLIENT_ERROR_EXPECTATION_FAILED, "Need POST Contents");
					return;
				}
				
				if(item.instanceOf("xnat:imageAssessorData")){
					XnatImageassessordata assessor = (XnatImageassessordata)BaseElement.GetGeneratedItem(item);
					
					//MATCH PROJECT
					if(this.proj==null && assessor.getProject()!=null){
						proj = XnatProjectdata.getXnatProjectdatasById(assessor.getProject(), user, false);
					}
					
					if(this.proj!=null){
						if(assessor.getProject()==null || assessor.getProject().equals("")){
							assessor.setProject(this.proj.getId());
						}else if(assessor.getProject().equals(this.proj.getId())){
						}else{
							boolean matched=false;
							for(XnatExperimentdataShare pp : assessor.getSharing_share()){
								if(pp.getProject().equals(this.proj.getId())){
									matched=true;
									break;
								}
							}
							
							if(!matched){
								XnatExperimentdataShare pp= new XnatExperimentdataShare((UserI)user);
								pp.setProject(this.proj.getId());
								assessor.setSharing_share(pp);
							}
						}
					}else{
						this.getResponse().setStatus(Status.CLIENT_ERROR_UNPROCESSABLE_ENTITY,"Submitted subject record must include the project attribute.");
						return;
					}
					
					//MATCH SESSION
					if(this.assessed!=null){
						assessor.setImagesessionId(this.assessed.getId());
					}else{
						if(assessor.getImagesessionId()!=null && !assessor.getImagesessionId().equals("")){
							this.assessed=XnatExperimentdata.getXnatExperimentdatasById(assessor.getImagesessionId(), user, false);
							
							if(this.assessed==null && assessor.getProject()!=null && assessor.getLabel()!=null){
							this.assessed=XnatExperimentdata.GetExptByProjectIdentifier(assessor.getProject(), assessor.getImagesessionId(),user, false);
							}
							
							if(this.assessed==null){
								for(XnatExperimentdataShare pp : assessor.getSharing_share()){
								this.assessed=XnatExperimentdata.GetExptByProjectIdentifier(pp.getProject(), assessor.getImagesessionId(),user, false);
									if(this.assessed!=null){
										break;
									}
								}
							}
						}
					}

					//FIND PRE-EXISTING
					XnatImageassessordata existing=null;
					if(assessor.getId()!=null){
						existing=(XnatImageassessordata)XnatExperimentdata.getXnatExperimentdatasById(assessor.getId(), user, completeDocument);
					}
					
					if(existing==null && assessor.getProject()!=null && assessor.getLabel()!=null){
					existing=(XnatImageassessordata)XnatExperimentdata.GetExptByProjectIdentifier(assessor.getProject(), assessor.getLabel(),user, completeDocument);
					}
					
					if(existing==null){
						for(XnatExperimentdataShare pp : assessor.getSharing_share()){
						existing=(XnatImageassessordata)XnatExperimentdata.GetExptByProjectIdentifier(pp.getProject(), pp.getLabel(),user, completeDocument);
							if(existing!=null){
								break;
							}
						}
					}
					
					if(existing==null){
						if(!user.canCreate(assessor)){
						this.getResponse().setStatus(Status.CLIENT_ERROR_FORBIDDEN,"Specified user account has insufficient create priviledges for subjects in this project.");
						return;
						}
						//IS NEW
						if(assessor.getId()==null || assessor.getId().equals("")){
						assessor.setId(XnatExperimentdata.CreateNewID());
						}
					}else{
						this.getResponse().setStatus(Status.CLIENT_ERROR_CONFLICT,"Specified experiment already exists.");
					return;
						//MATCHED
					}
					
					boolean allowDataDeletion=false;
					if(this.getQueryVariable("allowDataDeletion")!=null && this.getQueryVariable("allowDataDeletion").equals("true")){
						allowDataDeletion=true;
					}
					
				if(!StringUtils.IsEmpty(assessor.getLabel()) && !StringUtils.IsAlphaNumericUnderscore(assessor.getId())){
					this.getResponse().setStatus(Status.CLIENT_ERROR_EXPECTATION_FAILED,"Invalid character in experiment label.");
					return;
				}
				
					
				
				final ValidationResults vr = assessor.validate();
	            
	            if (vr != null && !vr.isValid())
	            {
	            	this.getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST,vr.toFullString());
					return;
	            }
				
				if(assessor.save(user,false,allowDataDeletion)){
					MaterializedView.DeleteByUser(user);
				}

				if(this.getQueryVariable("activate")!=null && this.getQueryVariable("activate").equals("true")){
					if(user.canActivate(assessor.getItem()))assessor.activate(user);
					else this.getResponse().setStatus(Status.CLIENT_ERROR_FORBIDDEN,"Specified user account has insufficient activation priviledges for experiments in this project.");
				}

				if(this.getQueryVariable("quarantine")!=null && this.getQueryVariable("quarantine").equals("true")){
					if(user.canActivate(assessor.getItem()))assessor.quarantine(user);
					else this.getResponse().setStatus(Status.CLIENT_ERROR_FORBIDDEN,"Specified user account has insufficient activation priviledges for experiments in this project.");
				}
				
				this.returnSuccessfulCreateFromList(assessor.getId());
				}else{
					this.getResponse().setStatus(Status.CLIENT_ERROR_UNPROCESSABLE_ENTITY,"Only xnat:Subject documents can be PUT to this address.");
				}
		} catch (InvalidValueException e) {
			this.getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			logger.error("",e);
			} catch (Exception e) {
				this.getResponse().setStatus(Status.SERVER_ERROR_INTERNAL);
			logger.error("",e);
			}
		}

	
	@Override
	public ArrayList<String> getDefaultFields(GenericWrapperElement e) {
		ArrayList<String> al=new ArrayList<String>();
		
		al.add("ID");
		al.add("project");
		al.add("label");
		al.add("date");
		al.add("xsiType");
		al.add("insert_date");

		if(e.instanceOf("xnat:imageAssessorData")){
			al.add("session_ID");
			al.add("session_label");
	}

		return al;
	}

	public String getDefaultElementName(){
		return "xnat:imageAssessorData";
	}

	@Override
	public Representation getRepresentation(Variant variant) {	
		XFTTable table = null;
		if(proj!=null && assessed!=null){
			Representation rep=super.getRepresentation(variant);
			if(rep!=null)return rep;
			
			try {
				String rootElementName=this.getRootElementName();
				QueryOrganizer qo = new QueryOrganizer(rootElementName,user,ViewManager.ALL);
				
				this.populateQuery(qo);
				
				CriteriaCollection where=new CriteriaCollection("AND");
					
					CriteriaCollection cc= new CriteriaCollection("OR");
					cc.addClause("xnat:imageAssessorData/imagesession_id", assessed.getId());
					where.addClause(cc);
					
				CriteriaCollection projects=new CriteriaCollection("OR");
				List<Object> ps=user.getAllowedValues("xnat:subjectData", "xnat:subjectData/project", org.nrg.xdat.security.SecurityManager.READ);
				for(Object p:ps){					
					projects.addClause(rootElementName+"/project", p);
					projects.addClause(rootElementName+"/sharing/share/project", p);
				}
				where.addClause(projects);
				
				qo.setWhere(where);
				
				String query=qo.buildQuery();
				
				table=XFTTable.Execute(query, user.getDBName(), userName);
				
				if(table.size()>0){
					table=formatHeaders(table,qo,rootElementName+"/ID","/REST/experiments/");
					
					final Integer labelI=table.getColumnIndex("label");
					final Integer idI=table.getColumnIndex(rootElementName+"/ID");
					if(labelI!=null && idI!=null){
						final XFTTable t= XFTTable.Execute("SELECT sharing_share_xnat_experimentda_id as id,label FROM xnat_experimentData_share WHERE project='"+ proj.getId() + "'", user.getDBName(), user.getUsername());
						final Hashtable lbls=t.toHashtable("id", "label");
						for(Object[] row:table.rows()){
							final String id=(String)row[idI];
							if(lbls.containsKey(id)){
								final String lbl=(String)lbls.get(id);
								if(null!=lbl && !lbl.equals("")){
									row[labelI]=lbl;
								}
							}
						}
					}
				}
			} catch (SQLException e) {
				logger.error("", e);
			} catch (DBPoolException e) {
				logger.error("", e);
			} catch (Exception e) {
				logger.error("", e);
			}

			Hashtable<String, Object> params = new Hashtable<String, Object>();
			if (table != null)
				params.put("totalRecords", table.size());
			return this.representTable(table, overrideVariant(variant), params);
			
		}
		
		Hashtable<String,Object> params=new Hashtable<String,Object>();
		params.put("title", "Project Subject Experiment Assessors");

		if(table!=null)params.put("totalRecords", table.size());
		return this.representTable(table,  overrideVariant(variant), params);
	}
}
